from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)


class rgb_lcd(ArduinoObject):

    def __init__(self, connection=None):
        ArduinoObject.__init__(self, connection=connection)
        self.id = self.call('new')

    @arduinoobjectmethod
    def clear(self):
        pass

    @arduinoobjectmethod
    def moveCursor(self, col, row):
        pass

    @arduinoobjectmethod
    def setRGB(self, red, blue, green):
        pass

    @arduinoobjectmethod
    def print(self, value):
        pass

    @arduinoobjectmethod
    def printString(self, value):
        pass



udistance_instance = None

def get_udistance(pin):
    """Get static reference to ultrasonic distance sensor.

    Parameters:
      pint (int):       Digital connection port (range: [2, 8])
    """

    global udistance_instance

    if udistance_instance is None:
        from . import arduino, ultrasonic

        udistance_instance = ultrasonic.Ultrasonic(pin, arduino().connection)

    return udistance_instance

def get_distance(pin):
    """Find sensed distance in cm

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
    """

    return get_udistance(pin).get_distance()



led_bar_instance = None

def get_led_bar(pin):
    """Get static reference to servo motor.

    Parameters:
      pint (int):       Digital connection port (range: [2, 8])
    """

    global led_bar_instance

    if led_bar_instance is None:
        from . import arduino, bargraph

        led_bar_instance = bargraph.led_bar(pin, arduino().connection)

    return led_bar_instance

def set_led_level(pin, level):
    """Light up number of bars indicated

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
      level (int):      Bars to illuminate (range: [0 10])
    """

    get_led_bar(pin).barSetLevel(level)


def set_led_light(pin, led, brightness):
    """Set brightness level of individual bar

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
      led (int):       Which bars to control (range: [0 10])
      brightness (float):       Level of brightness for bar (range: [0 1])

"""

    get_led_bar(pin).barSetLed(led, brightness)



rgba_leds_instance = None

def get_rgba_leds(pin, count):
    """Get static reference to chain of RGB LEDs

    Parameters:
      pint (int):       Digital connection port (range: [2, 8])
      count (int):      Number of LEDs connected
    """

    global rgba_leds_instance

    if rgba_leds_instance is None:
        from . import arduino, chainable_leds

        rgba_leds_instance = chainable_leds.rgba_leds(pin, count, arduino().connection)

    return rgba_leds_instance

def set_led_rgb(pin, count, led, red, green, blue):
    """Set red, green and blue level
        of individual LED

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
      count (int):     Number of LEDs connected in chain
      led (int):       Which LED to control (range: [0 count-1])
      red (float):       Red in LED colour(range: [0 255])
      green (float):     Green in LED colour (range: [0 255])
      blue (float):      Blue in LED colour (range: [0 255])

"""
    get_rgba_leds(pin, count).rgbLedRGB(led, red, green, blue)


def set_led_hsb(pin, count, led, hue, saturation, brightness):
    """Set hue, saturation and brightness level
        of individual LED

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
      count (int):     Number of LEDs connected in chain
      led (int):       Which LED to control (range: [0 count-1])
      hue (float):       Hue/colour for LED (range: [0 1])
      saturation (float):       Level of saturation for LED (range: [0 1])
      brightness (float):       Level of brightness for LED (range: [0 1])

"""

    get_rgba_leds(pin, count).rgbLedHSB(led, hue, saturation, brightness)



motor_instance = None

def get_motor(pin):
    """Get static reference to servo motor.

    Parameters:
      pint (int):       Digital connection port (range: [2, 8])
    """

    global motor_instance

    if motor_instance is None:
        from . import arduino, servo

        motor_instance = servo.Servo(pin, arduino().connection)

    return motor_instance

def move(pin, angle):
    """Move servo motor to given angle (in degrees)

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
      angle (int):      Final servo position (range: [0 180])
    """

    get_motor(pin).write(angle)
    from time import sleep
    sleep(0.5)

def read(pin):
    """Check position of servo motor

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])

    Returns:         Servo motor position (0–180).
    """

    return get_motor(pin).read()
    



import math

def celsius(pin):
    """Return temperature in celsius from sensor

    Parameters:
      pin (int):     Analog pin to read from (e.g. 1 for A1)

    Returns:         Calculated temperature in Celsius
    """

    from . import arduino
    reading = float(arduino().analogRead(pin+14))
    R = 1023.0/reading-1.0
    R = 100000*R
    temperature = 1.0/(math.log10(R/100000)/4275+1/298.15)-273.15;
    return temperature




from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class Servo(ArduinoObject):
    cfg_h_name = 'USE_Servo'

    def __init__(self, pin, connection=None):
        ArduinoObject.__init__(self, connection=connection)
        self.id = self.call('new', pin)

    @arduinoobjectmethod
    def write(self, value):
        pass

    @returns(int)
    @arduinoobjectmethod
    def read(self):
        pass

    @arduinoobjectmethod
    def writeMicroseconds(self, value):
        pass

    @returns(int)
    @arduinoobjectmethod
    def readMicroseconds(self, value):
        pass

    @arduinoobjectmethod
    def attached(self):
        pass

    @arduinoobjectmethod
    def detach(self):
        pass



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import arduinoobjectmethod

class Tone(ArduinoObject):

    cfg_h_name = 'USE_Tone'

    NOTE_B0  = 31
    NOTE_C1  = 33
    NOTE_CS1 = 35
    NOTE_D1  = 37
    NOTE_DS1 = 39
    NOTE_E1  = 41
    NOTE_F1  = 44
    NOTE_FS1 = 46
    NOTE_G1  = 49
    NOTE_GS1 = 52
    NOTE_A1  = 55
    NOTE_AS1 = 58
    NOTE_B1  = 62
    NOTE_C2  = 65
    NOTE_CS2 = 69
    NOTE_D2  = 73
    NOTE_DS2 = 78
    NOTE_E2  = 82
    NOTE_F2  = 87
    NOTE_FS2 = 93
    NOTE_G2  = 98
    NOTE_GS2 = 104
    NOTE_A2  = 110
    NOTE_AS2 = 117
    NOTE_B2  = 123
    NOTE_C3  = 131
    NOTE_CS3 = 139
    NOTE_D3  = 147
    NOTE_DS3 = 156
    NOTE_E3  = 165
    NOTE_F3  = 175
    NOTE_FS3 = 185
    NOTE_G3  = 196
    NOTE_GS3 = 208
    NOTE_A3  = 220
    NOTE_AS3 = 233
    NOTE_B3  = 247
    NOTE_C4  = 262
    NOTE_CS4 = 277
    NOTE_D4  = 294
    NOTE_DS4 = 311
    NOTE_E4  = 330
    NOTE_F4  = 349
    NOTE_FS4 = 370
    NOTE_G4  = 392
    NOTE_GS4 = 415
    NOTE_A4  = 440
    NOTE_AS4 = 466
    NOTE_B4  = 494
    NOTE_C5  = 523
    NOTE_CS5 = 554
    NOTE_D5  = 587
    NOTE_DS5 = 622
    NOTE_E5  = 659
    NOTE_F5  = 698
    NOTE_FS5 = 740
    NOTE_G5  = 784
    NOTE_GS5 = 831
    NOTE_A5  = 880
    NOTE_AS5 = 932
    NOTE_B5  = 988
    NOTE_C6  = 1047
    NOTE_CS6 = 1109
    NOTE_D6  = 1175
    NOTE_DS6 = 1245
    NOTE_E6  = 1319
    NOTE_F6  = 1397
    NOTE_FS6 = 1480
    NOTE_G6  = 1568
    NOTE_GS6 = 1661
    NOTE_A6  = 1760
    NOTE_AS6 = 1865
    NOTE_B6  = 1976
    NOTE_C7  = 2093
    NOTE_CS7 = 2217
    NOTE_D7  = 2349
    NOTE_DS7 = 2489
    NOTE_E7  = 2637
    NOTE_F7  = 2794
    NOTE_FS7 = 2960
    NOTE_G7  = 3136
    NOTE_GS7 = 3322
    NOTE_A7  = 3520
    NOTE_AS7 = 3729
    NOTE_B7  = 3951
    NOTE_C8  = 4186
    NOTE_CS8 = 4435
    NOTE_D8  = 4699
    NOTE_DS8 = 4978

    def __init__(self, pin, connection=None):
        ArduinoObject.__init__(self, connection=connection)
        self.id = self.call('new', pin)

    @arduinoobjectmethod
    def play(self, note, duration):
        pass

    @arduinoobjectmethod
    def stop(self):
        pass



import math

def centimeters(pin):
    """Return distance in cm from sensor

    Parameters:
      pin (int):     Analog pin to read from (e.g. 1 for A1)

    Returns:         Calculated distance in cm
    """

    from . import arduino
    reading = float(arduino().analogRead(pin+1+14))
    distance = 27.726*((reading*5/1024)**-1.2045)
    return distance




buzzer_instance = None

def get_buzzer(pin):
    """Get static reference to buzzer.

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
    """

    global buzzer_instance

    if buzzer_instance is None:
        from . import arduino, tone

        buzzer_instance = tone.Tone(pin, arduino().connection)

    return buzzer_instance

def note(pin, freq, duration):
    """Play note of given frequency

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
      freq (int):      Frequency of note in Hz
      duration (int):  Note length in seconds
    """

    get_buzzer(pin).play(freq, duration)

def stop(pin):
    """Stop note

    Parameters:
      pin (int):       Digital connection port (range: [2, 8])
    """

    get_buzzer(pin).stop()



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class rgba_leds(ArduinoObject):
    cfg_h_name = 'USE_rgba_leds'

    def __init__(self, pin, count, connection=None):
        ArduinoObject.__init__(self, connection=connection)
        self.id = self.call('new', pin, count)

    @arduinoobjectmethod
    def rgbLedHSB(self, led, hue, saturation, brightness):
        pass

    @arduinoobjectmethod
    def rgbLedRGB(self, led, red, green, blue):
        pass



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class Ultrasonic(ArduinoObject):

    cfg_h_name = 'USE_ultrasonic_distance'

    def __init__(self, connection=None):
        ArduinoObject.__init__(self, connection=connection)
        self.id = self.call('new')

    @arduinoobjectmethod
    def ultraInit(self, port):
        pass
 
    @arduinoobjectmethod
    def get_distance(self):
        pass
